@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,9 +25,9 @@
 # Some version numbers and things.
 
 # ***** ***** ***** ***** ***** ***** **** ***** ***** ***** *****
-# Be sure to change script_version in data/script_version.rpy, too!
+# Be sure to change script_version in launcher/script_version.rpy, too!
 # Also check to see if we have to update renpy.py.
-version = "Ren'Py 6.7.1d"
+version = "Ren'Py 6.9.2a"
 script_version = 5003000
 savegame_suffix = "-LT1.save"
 
@@ -55,12 +55,15 @@ def import_all():
 
     import renpy.display
     import renpy.display.presplash
+    import renpy.display.iliad # Must be before scale.
     import renpy.display.scale # Must be before module.
     import renpy.display.module
     import renpy.display.render # Most display stuff depends on this.
     import renpy.display.core # object
-    import renpy.display.text # core
+    import renpy.display.font
+    import renpy.display.text # core, font
     import renpy.display.layout # core
+    import renpy.display.motion # layout
     import renpy.display.behavior # layout
     import renpy.display.transition # core, layout
     import renpy.display.im
@@ -76,8 +79,8 @@ def import_all():
     # Note: For windows to work, renpy.audio.audio needs to be after
     # renpy.display.module. 
     import renpy.audio.audio
-    import renpy.audio.sound
     import renpy.audio.music
+    import renpy.audio.sound
 
     import renpy.ui
 
@@ -106,8 +109,10 @@ def reload_all():
     import sys
     sys.meta_path.pop()
 
+    blacklist = [ "renpy", "renpy.bootstrap", "renpy.display", "renpy.display.iliad", "renpy.display.scale" ]
+    
     for i in sys.modules.keys():
-        if i.startswith("renpy") and i != "renpy" and i != "renpy.bootstrap":
+        if i.startswith("renpy") and i not in blacklist:
             del sys.modules[i]
 
     import gc
