@@ -34,13 +34,14 @@ def extra_imports():
     import encodings.string_escape
     import encodings.raw_unicode_escape
     import encodings.mbcs
+    import encodings.utf_16_be
+    import encodings.utf_16_le
     import math
     import glob
     import pickle
     import platform
     import pysdlsound
     import pysdlsound.sound
-    import pysdlsound.nativemidi
     import pysdlsound.winmixer
     import pysdlsound.linmixer
     import difflib
@@ -52,6 +53,7 @@ def extra_imports():
     import webbrowser
     import pygame.locals
     import pygame.color
+    import pygame.colordict
     
     
 trace_file = None
@@ -95,6 +97,14 @@ def bootstrap(renpy_base):
 
     global renpy
 
+    # If environment.txt exists, load it into the os.environ dictionary.
+    if os.path.exists(renpy_base + "/environment.txt"):
+        evars = { }
+        execfile(renpy_base + "/environment.txt", evars)
+        for k, v in evars.iteritems():
+            if k not in os.environ:
+                os.environ[k] = str(v)
+    
     # Get a working name for the game.
     name = os.path.basename(sys.argv[0])
 
@@ -246,6 +256,8 @@ def bootstrap(renpy_base):
             keep_running = False
 
         except KeyboardInterrupt:
+            import traceback
+            traceback.print_exc()
             break
             
         except renpy.game.UtterRestartException:
@@ -308,10 +320,12 @@ def report_exception(e):
     type, value, tb = sys.exc_info()
 
     def safe_utf8(e):
-        try:
-            return unicode(e).encode("utf-8")
-        except:
-            return str(e)
+        m = e.message
+        
+        if isinstance(m, unicode):
+            return m.encode("utf-8")
+        else:
+            return str(m)
 
     # Outside of the file.
     traceback.print_tb(tb, None, sys.stdout)
@@ -349,6 +363,7 @@ def report_exception(e):
         print >>f
 
         print >>f, renpy.game.exception_info
+        report_tb(f, tb)
 
         print >>f
         print >>f, "Ren'Py Version:", renpy.version
@@ -382,23 +397,14 @@ def memory_profile():
     objs = gc.get_objects()
 
     c = { } # count
-    dead_renders = 0
 
     for i in objs:
         t = type(i)
         c[t] = c.get(t, 0) + 1
 
-        if isinstance(i, renpy.display.render.Render):
-            if i.dead:
-                dead_renders += 1
-            
-
     results = [ (count, ty) for ty, count in c.iteritems() ]
     results.sort()
 
     for count, ty in results:
         print count, str(ty)
 
-    if dead_renders:
-        print
-        print "*** found", dead_renders, "dead Renders. ***"
