@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -54,6 +54,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
                      two_window_vbox_properties={},
                      who_window_properties={},
                      say_vbox_properties={},
+                     variant=None,
                      **kwargs):
     """
     This is called (by default) by renpy.display_say to add the
@@ -100,6 +101,9 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
         if isinstance(style, basestring):
             style = getattr(renpy.store.style, style)
 
+        if variant is not None:
+            style = style[variant]
+            
         if renpy.exports.in_rollback():
             style = style["rollback"]
             
@@ -112,7 +116,9 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
         if not "style" in d:
             return d
 
-        if not renpy.exports.in_rollback():
+        in_rollback = renpy.exports.in_rollback()
+        
+        if (not in_rollback) and (not variant):
             return d
             
         d = d.copy()
@@ -121,7 +127,12 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 
         if isinstance(style, basestring):
             style = getattr(renpy.store.style, style)        
-            style = style["rollback"]
+
+            if variant is not None:
+                style = style[variant]
+
+            if in_rollback:
+                style = style["rollback"]
 
         d["style"] = style
 
@@ -129,7 +140,8 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
     
     who_args = style_args(who_args)
     what_args = style_args(what_args)
-        
+    window_args = style_args(window_args)
+    
     if two_window:
 
         # Opens say_two_window_vbox.
@@ -157,6 +169,8 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
     if side_image:
         renpy.ui.image(side_image)
 
+    renpy.exports.shown_window()
+        
     return rv
 
 
@@ -196,11 +210,11 @@ def display_say(show_function,
                 type,
                 checkpoint=True):
 
-    ctc = renpy.easy.displayable(ctc)
+    ctc = renpy.easy.displayable_or_none(ctc)
     
     # If we're in fast skipping mode, don't bother with say
     # statements at all.
-    if renpy.config.skipping == "fast":
+    if interact and renpy.config.skipping == "fast":
 
         # Clears out transients.
         renpy.exports.with_statement(None)
@@ -218,6 +232,7 @@ def display_say(show_function,
 
     callback = renpy.config.all_character_callbacks + callback 
 
+    
     # Call the begin callback.
     for c in callback:
         c("begin", interact=interact, type=type, **cb_args)
@@ -288,7 +303,7 @@ def display_say(show_function,
         # Now, re-run update on what_text.
         what_text.update(retokenize=False)
             
-        keep_interacting, pause_length = what_text.get_keep_pausing()
+        keep_interacting = what_text.keep_pausing
         no_wait |= what_text.no_wait
 
         if no_wait:
@@ -300,14 +315,11 @@ def display_say(show_function,
         if behavior and afm:
             behavior.set_afm_length(what_text.get_simple_length() - slow_start)
 
-
         if interact:
-            if pause_length is not None:
-                renpy.ui.pausebehavior(pause_length, True)
             
             rv = renpy.ui.interact(mouse='say', type=type, roll_forward=roll_forward)
 
-            # This is only the case if the user has rolled forward, or
+            # This is only the case if the user has rolled forward, {nw} happens, or
             # maybe in some other obscure cases.
             if rv is False:
                 break 
@@ -407,8 +419,8 @@ class ADVCharacter(object):
             interact = d('interact'),
             slow = d('slow'),
             afm = d('afm'),
-            ctc = renpy.easy.displayable(d('ctc')),
-            ctc_pause = renpy.easy.displayable(d('ctc_pause')),
+            ctc = renpy.easy.displayable_or_none(d('ctc')),
+            ctc_pause = renpy.easy.displayable_or_none(d('ctc_pause')),
             ctc_position = d('ctc_position'),
             all_at_once = d('all_at_once'),
             with_none = d('with_none'),
@@ -416,7 +428,6 @@ class ADVCharacter(object):
             type = d('type'),
         )
 
-
         if kind:
             self.who_args = kind.who_args.copy()
             self.what_args = kind.what_args.copy()
