@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -32,16 +32,22 @@ import time
 import cStringIO
 import threading
 
+on_windows = (sys.platform == 'win32')
+
+try:
+    import pygame.macosx
+except:
+    pass
+
 # Is the cpu idle enough to do other things?
 cpu_idle = threading.Event()
 cpu_idle.clear()
 
-
-
-TIMEEVENT = USEREVENT + 1
-PERIODIC = USEREVENT + 2
-JOYEVENT = USEREVENT + 3
-REDRAW = USEREVENT + 4
+# Need to be +4, so we don't interfere with FFMPEG's events.
+TIMEEVENT = USEREVENT + 4
+PERIODIC = USEREVENT + 5
+JOYEVENT = USEREVENT + 6
+REDRAW = USEREVENT + 7
 
 # The number of msec between periodic events.
 PERIODIC_INTERVAL = 50
@@ -64,15 +70,12 @@ class IgnoreEvent(Exception):
 
     pass
 
+class absolute(float):
+    """
+    This represents an absolute float coordinate.
+    """
+    __slots__ = [ ]
 
-# Names for anchors.
-anchors = dict(
-    left=0.0,
-    right=1.0,
-    center=0.5,
-    top=0.0,
-    bottom=1.0,
-    )
 
 class Displayable(renpy.object.Object):
     """
@@ -116,7 +119,8 @@ class Displayable(renpy.object.Object):
             self.set_style_prefix(self.role + "hover_")
         
         if not default and not self.activated:
-            renpy.audio.sound.play(self.style.sound)
+            if self.style.sound:
+                renpy.audio.music.play(self.style.sound, channel="sound")
 
     def unfocus(self):
         """
@@ -269,7 +273,7 @@ class Displayable(renpy.object.Object):
         self.render().
         """
         
-        xpos, ypos, xanchor, yanchor, xoffset, yoffset = self.get_placement()
+        xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel  = self.get_placement()
 
         if xpos is None:
             xpos = 0
@@ -286,40 +290,45 @@ class Displayable(renpy.object.Object):
         if xoff is None:
             xoff = xpos
 
+        if type(xoff) is float:
+            xoff_mul = width
+        else:
+            xoff_mul = 1
 
-        if isinstance(xoff, float):
-            xoff = int(xoff * width)
-
-        if isinstance(xanchor, int):
-            xoff -= xanchor
-        else:            
-            xanchor = anchors.get(xanchor, xanchor)
-            xoff -= int(sw * xanchor)
+        if type(xanchor) is float:
+            xanchor_mul = sw
+        else:
+            xanchor_mul = 1
 
+        xoff = xoff * xoff_mul - xanchor * xanchor_mul
         xoff += x
 
         # y
-
         if yoff is None:
             yoff = ypos
 
-        if isinstance(yoff, float):
-            yoff = int(yoff * height)
+        if type(yoff) is float:
+            yoff_mul = height
+        else:
+            yoff_mul = 1
 
-        if isinstance(yanchor, int):
-            yoff -= yanchor
-        else:            
-            yanchor = anchors.get(yanchor, yanchor)
-            yoff -= int(sh * yanchor)
+        if type(yanchor) is float:
+            yanchor_mul = sh
+        else:
+            yanchor_mul = 1
 
+        yoff = yoff * yoff_mul - yanchor * yanchor_mul            
         yoff += y
 
         # Add in offsets.
         xoff += xoffset
         yoff += yoffset
-        
-        dest.blit(surf, (xoff, yoff), main=main)
 
+        if subpixel:
+            dest.subpixel_blit(surf, (xoff, yoff), main=main)
+        else:
+            dest.blit(surf, (xoff, yoff), main=main)
+            
         return xoff, yoff
 
 class ImagePredictInfo(renpy.object.Object):
@@ -373,7 +382,7 @@ class SceneLists(renpy.object.Object):
     things to the user. 
     """
 
-    __version__ = 1
+    __version__ = 2
     
     def after_setstate(self):
         for i in renpy.config.layers + renpy.config.top_layers:
@@ -391,7 +400,6 @@ class SceneLists(renpy.object.Object):
             for i in renpy.config.layers + renpy.config.top_layers:
                 self.at_list[i] = { }
                 self.layer_at_list[i] = (None, [ ])
-                
 
     def __init__(self, oldsl, ipi):
 
@@ -419,7 +427,6 @@ class SceneLists(renpy.object.Object):
 
             self.replace_transient()
 
-            self.movie = oldsl.movie
             self.focused = None
             
         else:
@@ -429,7 +436,6 @@ class SceneLists(renpy.object.Object):
                 self.layer_at_list[i] = (None, [ ])
                 
             self.music = None
-            self.movie = None
             self.focused = None
 
     def replace_transient(self):
@@ -658,6 +664,9 @@ class Display(object):
         # Ensure that we kill off the movie when changing screen res.
         renpy.display.video.movie_stop(clear=False)
 
+        if pygame.version.vernum < (1, 8, 1):
+            raise Exception("Ren'Py requires pygame 1.8.1 to run.")
+        
         try:
             pygame.macosx.init()
         except:
@@ -665,11 +674,41 @@ class Display(object):
 
         pygame.display.init()
         pygame.font.init()
+
+        renpy.display.scale.init()
+
         renpy.audio.audio.init()
         renpy.display.joystick.init()
 
         init_time()
         
+        # Window icon.
+
+        icon = renpy.config.window_icon
+
+        if on_windows and renpy.config.windows_icon:
+            icon = renpy.config.windows_icon
+            
+        if icon:
+
+            im = renpy.display.scale.image_load_unscaled(
+                renpy.loader.load(icon),
+                icon,
+                convert=False,
+                )
+
+            # Convert the aspect ratio to be square.
+            iw, ih = im.get_size()
+            imax = max(iw, ih)
+            square_im = renpy.display.scale.PygameSurface((imax, imax), pygame.SRCALPHA, depth=32)
+            square_im.blit(im, ( (imax-iw)/2, (imax-ih)/2 ))
+            im = square_im
+
+            if on_windows and im.get_size() != (32, 32):
+                im = renpy.display.scale.real_smoothscale(im, (32, 32))
+                
+            pygame.display.set_icon(im)
+
         # Setup screen.
         fullscreen = renpy.game.preferences.fullscreen
 
@@ -685,36 +724,39 @@ class Display(object):
             fullscreen = False
             self.fullscreen = renpy.game.preferences.fullscreen
 
-#         if fullscreen == "16:9":
-#             fsratio = 16.0 / 9.0
-#         elif fullscreen == "16:10":
-#             fsratio = 16.0 / 10.0
-#         elif fullscreen: # 4:3 mode.
-#             fsratio = 4.0 / 3.0
-#         else:
-#             fsratio = None
-
-        if fullscreen:
-            fsflag = FULLSCREEN
-#             width = int(max(renpy.config.screen_width, renpy.config.screen_height * fsratio))
-#             height = int(max(renpy.config.screen_height, renpy.config.screen_width / fsratio))
-#             self.screen_xoffset = (width - renpy.config.screen_width) / 2
-#             self.screen_yoffset = (height - renpy.config.screen_height) / 2            
-        else:
-            fsflag = 0
 
         width = renpy.config.screen_width
         height = renpy.config.screen_height
         self.screen_xoffset = 0
         self.screen_yoffset = 0
-            
-        # Window icon.
-        if renpy.config.window_icon:
-            pygame.display.set_icon(
-                renpy.display.scale.image_load_unscaled(
-                    renpy.loader.load(renpy.config.window_icon),
-                    renpy.config.window_icon))
+        fsflag = 0
 
+        if fullscreen == "wide" or fullscreen == "narrow":
+            for w, h in pygame.display.list_modes():
+                ratio = 1.0 * w / h
+                if ratio >= 2:
+                    continue
+
+                if fullscreen == "wide":
+                    if ratio < 1.5:
+                        continue
+
+                elif fullscreen == "narrow":
+                    if ratio >= 1.5:
+                        continue
+
+                if w < renpy.config.screen_width or h < renpy.config.screen_height:
+                    continue
+                
+                fsflag = FULLSCREEN
+                width = w
+                height = h
+                self.screen_xoffset = (width - renpy.config.screen_width) / 2
+                self.screen_yoffset = (height - renpy.config.screen_height) / 2
+
+        elif fullscreen:
+            fsflag = FULLSCREEN
+              
         # If a window exists of the right size and flags, use it. Otherwise,
         # make our own window.
         old_window = pygame.display.get_surface()
@@ -727,13 +769,15 @@ class Display(object):
         else:
             self.window = pygame.display.set_mode((width, height), fsflag, 32)
 
+                
+
         # Window title.
         self.window_caption = None
         self.set_window_caption()
         
         # Sample surface that all surfaces are created based on.
-        sample = pygame.Surface((10, 10))
-        self.sample_surface = sample.convert_alpha()
+        sample = pygame.Surface((10, 10), 0, self.window)
+        self.sample_surface = sample.convert_alpha(self.window)
 
         pygame.event.set_grab(False)
         
@@ -774,6 +818,9 @@ class Display(object):
         self.window_caption = caption
         pygame.display.set_caption(caption.encode("utf-8"))
 
+    def iconify(self):
+        pygame.display.iconify()
+        
     def can_redraw(self, first_pass):
         """
         Uses the framerate to determine if we can and should redraw.
@@ -787,7 +834,7 @@ class Display(object):
         next_frame = self.next_frame
         now = pygame.time.get_ticks()
 
-        frametime = 1000 / framerate
+        frametime = 1000.0 / framerate
 
         # Handle timer rollover.
         if next_frame > now + frametime:
@@ -798,10 +845,10 @@ class Display(object):
             return False
             
         # Otherwise, it is. Schedule the next frame.
-        if next_frame + frametime < now:
-            next_frame = now + frametime
-        else:
-            next_frame += frametime
+        # if next_frame + frametime < now:
+        next_frame = now + frametime
+        # else:
+        #    next_frame += frametime
 
         self.next_frame = next_frame
 
@@ -865,7 +912,7 @@ class Display(object):
         if self.mouse_event_time + renpy.config.mouse_hide_time < get_time():
             visible = False
         else:
-            visible = renpy.store.mouse_visible
+            visible = renpy.store.mouse_visible and (not renpy.game.less_mouse)
             
         # Deal with a hardware mouse, the easy way.
         if not self.mouse:
@@ -927,24 +974,14 @@ class Display(object):
         
     def show(self, root_widget, suppress_blit):
         """
-        Draws the current transient screen list to the screen.
-        
-        @returns A list of offsets corresponding to each widget,
-        relative to the screen.
+        Draws the screen.
         """
-
-#         if self.next_draw < pygame.time.get_ticks():
-#             self.next_draw = pygame.time.get_ticks()
-
-#         pygame.time.delay(self.next_draw - pygame.time.get_ticks())
-
-#         self.next_draw += 1000 / 30
-
+        
         surftree = renpy.display.render.render_screen(
             root_widget,
             renpy.config.screen_width,
             renpy.config.screen_height,
-            0)
+            )
 
         if not suppress_blit:
 
@@ -952,7 +989,7 @@ class Display(object):
 
             updates.extend(self.draw_mouse(False))
 
-            damage = renpy.display.render.screen_blit(surftree, self.full_redraw, self.screen_xoffset, self.screen_yoffset)
+            damage = renpy.display.render.draw_screen(self.screen_xoffset, self.screen_yoffset, self.full_redraw)
 
             if damage:
                 updates.extend(damage)
@@ -961,14 +998,14 @@ class Display(object):
 
             updates.extend(self.draw_mouse(True))
             pygame.display.update(updates)
-
+            
         else:
             self.full_redraw = True
-            
+
+        renpy.display.render.kill_old_screen()
+        renpy.display.focus.take_focuses()
+        
         self.suppressed_blit = suppress_blit
-
-        renpy.display.focus.take_focuses(surftree.focuses)
-
         self.surftree = surftree
         
     def save_screenshot(self, filename):
@@ -976,23 +1013,26 @@ class Display(object):
         Saves a full-size screenshot in the given filename.
         """
 
-        if filename.endswith(".png"):
-            f = file(filename, "wb")
-            renpy.display.module.save_png(self.window, f)
-            f.close()
-        else:
-            pygame.image.save(self.window, filename)
-
+        try:
+            renpy.display.scale.image_save_unscaled(self.window, filename)
+        except:
+            if renpy.config.debug:
+                raise
+            pass
+        
+            
+        
     def screenshot(self, scale):
         """
-        Returns a pygame Surface that is a screenshot of the current
-        contents of the window.
+        Returns a string containing the contents of the window, as a PNG.
         """
 
-        surf = renpy.display.module.scale(self.window, scale)
+        surf = self.window.convert_alpha()
+        surf = renpy.display.scale.smoothscale(surf, scale)
+        surf = surf.convert()
         
         sio = cStringIO.StringIO()
-        pygame.image.save(surf, sio)
+        renpy.display.module.save_png(surf, sio, 0)
         rv = sio.getvalue()
         sio.close()
         
@@ -1088,11 +1128,12 @@ class Interface(object):
 
         # Are we focused?
         self.focused = True
-
         
         # Properties for each layer.
         self.layer_properties = { }
 
+        # Have we shown the window this interaction?
+        self.shown_window = False
         
         for layer in renpy.config.layers + renpy.config.top_layers:
             if layer in renpy.config.layer_clipping:
@@ -1143,6 +1184,16 @@ class Interface(object):
 
         self.screenshot = None
 
+    def show_window(self):
+
+        if not renpy.store._window:
+            return
+
+        if self.shown_window:
+            return
+
+        if renpy.config.empty_window:
+            renpy.config.empty_window()
 
     def do_with(self, trans, paired, clear=False):
         
@@ -1164,6 +1215,8 @@ class Interface(object):
         Implements the with None command, which sets the scene we will
         be transitioning from.
         """
+
+        self.show_window()
         
         scene_lists = renpy.game.context().scene_lists
 
@@ -1177,7 +1230,8 @@ class Interface(object):
 
         scene_lists = renpy.game.context().scene_lists
         scene_lists.replace_transient()
-
+        self.shown_window = False
+        
     def set_transition(self, transition, layer=None, force=False):
         """
         Sets the transition that will be performed as part of the next
@@ -1233,14 +1287,6 @@ class Interface(object):
             self.pushed_event = None
             self.last_event = rv
             return rv
-
-#         while renpy.display.im.cache.needs_preload():
-#             ev = pygame.event.poll()
-#             if ev.type != NOEVENT:
-#                 self.last_event = ev
-#                 return ev
-
-#             renpy.display.im.cache.preload()
             
         try:
             cpu_idle.set()            
@@ -1272,14 +1318,30 @@ class Interface(object):
         rv[None] = root
              
         return rv
-            
 
-    def interact(self, clear=True, **kwargs):
+
+    def quit_event(self):
+        """
+        This is called to handle the user invoking a quit.
+        """
+        
+        if renpy.game.script.has_label("_confirm_quit") and not self.quick_quit:
+            self.quick_quit = True
+            renpy.game.call_in_new_context("_confirm_quit")
+            self.quick_quit = False
+        else:                
+            raise renpy.game.QuitException()
+
+    def interact(self, clear=True, suppress_window=False, **kwargs):
         """
         This handles an interaction, restarting it if necessary. All of the
         keyword arguments are passed off to interact_core.
         """
 
+        # Show a missing window.
+        if not suppress_window:
+            self.show_window()
+        
         # These things can be done once per interaction.
 
         import time
@@ -1313,6 +1375,8 @@ class Interface(object):
             self.transition_from = { }
                 
             self.restart_interaction = True
+
+            self.shown_window = False
             
     def interact_core(self,
                       show_mouse=True,
@@ -1339,7 +1403,7 @@ class Interface(object):
         """
         
         suppress_overlay = suppress_overlay or renpy.store.suppress_overlay        
-        suppress_transition = renpy.config.skipping
+        suppress_transition = renpy.config.skipping or renpy.game.less_updates
 
         # The global one.
         self.suppress_transition = False
@@ -1452,7 +1516,7 @@ class Interface(object):
             focus_roots.append(scene_layer)
 
             if (self.ongoing_transition.get(layer, None) and
-                 not suppress_transition):
+                not suppress_transition):
 
                 trans = self.ongoing_transition[layer](
                     old_widget=self.transition_from[layer],
@@ -1478,8 +1542,20 @@ class Interface(object):
         if (None in self.ongoing_transition and
             not suppress_transition):
 
+            old_root = renpy.display.layout.MultiBox(layout='fixed')
+            old_root.layers = { }
+
+            for layer in renpy.config.layers:
+                if layer in renpy.config.overlay_layers:
+                    d = scene[layer]
+                else:                
+                    d = self.transition_from[None].layers[layer]
+
+                old_root.layers[layer] = d
+                old_root.add(d)
+            
             trans = self.ongoing_transition[None](
-                old_widget=self.transition_from[None],
+                old_widget=old_root,
                 new_widget=layers_root)
 
             if not isinstance(trans, Displayable):
@@ -1505,6 +1581,13 @@ class Interface(object):
         for layer in renpy.config.top_layers:
             add_layer(root_widget, layer)
 
+        # Clean out the registered adjustments.
+        renpy.display.behavior.adj_registered.clear()
+
+        # Clean up some movie-related things.
+        renpy.display.video.early_interact()
+
+        
         # Call per-interaction code for all widgets.
         root_widget.visit_all(lambda i : i.per_interact())
         
@@ -1567,7 +1650,7 @@ class Interface(object):
                     suppress_blit = renpy.display.video.interact()
 
                     # Clean out the redraws, if we have to.
-                    renpy.display.render.kill_redraws()
+                    # renpy.display.render.kill_redraws()
                     
                     # Draw the screen.
                     self.frame_time = get_time()
@@ -1696,7 +1779,6 @@ class Interface(object):
 
                     if renpy.config.profile:
                         self.profile_time = get_time()
-
                     
                     # Try to merge an TIMEEVENT with the next event.
                     if ev.type == TIMEEVENT:
@@ -1726,7 +1808,10 @@ class Interface(object):
                         renpy.audio.audio.periodic()
                         continue
 
-                    
+                    # Handle ffdecode events.
+                    if renpy.audio.audio.event(ev):
+                        continue
+                                            
                     # This checks the event to see if it's a mouse event,
                     # and updates the mouse event timer as appropriate.
                     self.display.mouse_event(ev)
@@ -1741,12 +1826,7 @@ class Interface(object):
                     
                     # Handle quit specially for now.
                     if ev.type == QUIT:
-                        if renpy.game.script.has_label("_confirm_quit") and not self.quick_quit:
-                            self.quick_quit = True
-                            renpy.game.call_in_new_context("_confirm_quit")
-                            self.quick_quit = False
-                        else:                
-                            raise renpy.game.QuitException()
+                        self.quit_event()
 
                     # Merge mousemotion events.
                     if ev.type == MOUSEMOTION:
@@ -1754,7 +1834,8 @@ class Interface(object):
                         if len(evs):
                             ev = evs[-1]
 
-                        self.focused = True
+                        if on_windows:
+                            self.focused = True
                             
                     # Handle focus notifications.
                     if ev.type == ACTIVEEVENT:
