@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -49,7 +49,7 @@ class UncachedImage(renpy.display.core.Displayable):
         super(UncachedImage, self).__init__(style=style, **properties)
 
         self.surf = pygame.image.load(file, hint)
-        self.surf = self.surf.convert_alpha()
+        self.surf = self.surf.convert_alpha(renpy.game.interface.display.window)
 
         if scale:
             renpy.display.render.blit_lock.acquire()
@@ -92,7 +92,7 @@ class ImageReference(renpy.display.core.Displayable):
 
         if isinstance(name, renpy.display.core.Displayable):
             self.target = name
-            return
+            return True
         
         if not isinstance(name, tuple):
             name = tuple(name.split())
@@ -120,15 +120,16 @@ class ImageReference(renpy.display.core.Displayable):
 
                     error(str(e))
 
-                return
+                return True
 
             else:
                 parameters.insert(0, name[-1])
                 name = name[:-1]
 
         error("Image '%s' not found." % ' '.join(self.name))
+        return False
+        
         
-
     def event(self, ev, x, y, st):
         if not self.target:
             self.find_target()
@@ -145,8 +146,8 @@ class ImageReference(renpy.display.core.Displayable):
         if not self.target:
             self.find_target()
 
-        xpos, ypos, xanchor, yanchor, xoffset, yoffset = self.target.get_placement()
-        
+        xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel = self.target.get_placement()
+
         if xpos is None:
             xpos = self.style.xpos
 
@@ -158,8 +159,8 @@ class ImageReference(renpy.display.core.Displayable):
 
         if yanchor is None:
             yanchor = self.style.yanchor
-
-        return xpos, ypos, xanchor, yanchor, xoffset, yoffset
+            
+        return xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel
 
     def visit(self):
         if not self.target:
