@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,23 +31,35 @@ proc = None
 # user. If it decides to show something to the user, uses subprocess
 # to actually handle the showing.
 def start(gamedir):
+    import os
     import os.path
 
-    global proc
-    
-    fn = gamedir + "/presplash.png"
-    
-    if not os.path.exists(fn):
+    if "RENPY_LESS_UPDATES" in os.environ:
         return
-        
-    import subprocess
-    import sys
+    
+    global proc
 
-    if sys.argv[0].lower().endswith(".exe"):
-        proc = subprocess.Popen([sys.argv[0], "--presplash", fn], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+    filenames = [ "/presplash.png", "/presplash.jpg" ]
+    for fn in filenames:
+        fn = gamedir + fn
+        if os.path.exists(fn):
+            break
     else:
-        proc = subprocess.Popen([sys.executable, sys.argv[0], "--presplash", fn], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
-        
+        return
+    
+
+    try:    
+        import subprocess
+        import sys
+
+        if sys.argv[0].lower().endswith(".exe"):
+            proc = subprocess.Popen([sys.argv[0], "--presplash", fn], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+        else:
+            proc = subprocess.Popen([sys.executable, sys.argv[0], "--presplash", fn], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+    except:
+        pass
+            
+            
 # Called just before we initialize the display for real, to
 # hide the splash, and terminate window centering.
 def end():
