@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -62,6 +62,19 @@ class Transition(renpy.display.core.Displayable):
     def visit(self):
         return [ self.new_widget, self.old_widget ]
 
+def null_render(d, width, height, st, at):
+
+    d.events = True
+    surf = renpy.display.render.render(d.new_widget,
+                                       width,
+                                       height,
+                                       st, at)
+
+    rv = renpy.display.render.Render(surf.width, surf.height)
+    rv.blit(surf, (0, 0))
+    
+    return rv
+    
 class NoTransition(Transition):
     """
     This is a transition that doesn't do anything, and simply displays
@@ -78,19 +91,7 @@ class NoTransition(Transition):
         self.events = True
 
     def render(self, width, height, st, at):
-
-        surf = renpy.display.render.render(self.new_widget,
-                                           width,
-                                           height,
-                                           st, at)
-
-        width = min(surf.width, width)
-        height = min(surf.height, height)
-        
-        rv = renpy.display.render.Render(width, height)
-        rv.blit(surf, (0, 0))
-
-        return rv
+        return null_render(self, width, height, st, at)
 
 
 class MultipleTransition(Transition):
@@ -147,7 +148,10 @@ class MultipleTransition(Transition):
         return [ i for i in self.screens if isinstance(i, renpy.display.core.Displayable)] + self.transitions
 
     def render(self, width, height, st, at):
-        
+
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
+
         while True:
             trans = self.transitions[0]
             stoff = st - self.time_offset
@@ -239,6 +243,11 @@ class Pixellate(Transition):
     steps in each direction.
     """
 
+    nosave = [ 'surface', 'surface_size' ]
+                   
+    surface = None
+    surface_size = None
+    
     def __init__(self, time, steps, old_widget=None, new_widget=None, **properties):
 
         time = float(time)
@@ -254,15 +263,15 @@ class Pixellate(Transition):
         self.old_widget = old_widget
         self.new_widget = new_widget
 
-        self.surface = None
-        self.surface_size = None
-
         self.events = False
 
         self.quantum = time / ( 2 * steps )
 
     def render(self, width, height, st, at):
 
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
+
         if st >= self.time:
             self.events = True
             return render(self.new_widget, width, height, st, at)
@@ -277,7 +286,6 @@ class Pixellate(Transition):
 
         rdr = render(visible, width, height, st, at)
 
-        # No alpha support.
         surf = rdr.pygame_surface(False)
         
         if surf.get_size() != self.surface_size:
@@ -291,12 +299,8 @@ class Pixellate(Transition):
 
         rv = renpy.display.render.Render(rdr.width, rdr.height)
         rv.blit(self.surface, (0, 0))
-
-        if self.events:
-            rv.focuses.extend(rdr.focuses)
-
-        # renpy.display.render.redraw(self, self.quantum - st % self.quantum)
-
+        rv.depends_on(rdr)
+        
         renpy.display.render.redraw(self, 0)
 
         return rv
@@ -328,6 +332,9 @@ class Dissolve(Transition):
 
 
     def render(self, width, height, st, at):
+
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
         
         if st >= self.time or not renpy.display.module.can_blend:
             self.events = True
@@ -340,7 +347,7 @@ class Dissolve(Transition):
 
         bottom = render(self.old_widget, width, height, st, at)
         top = render(self.new_widget, width, height, st, at)
-
+        
         bottom_surface = bottom.pygame_surface(self.alpha)
         top_surface = top.pygame_surface(self.alpha)
 
@@ -348,7 +355,7 @@ class Dissolve(Transition):
         height = min(top.height, bottom.height)
 
         def draw(dest, x, y):
-
+            
             dw, dh = dest.get_size()
 
             w = min(dw, width + x)
@@ -363,6 +370,7 @@ class Dissolve(Transition):
                 dest.subsurface((0, 0, w, h)),
                 alpha)
 
+
         if self.alpha:
 
             rv = renpy.display.render.Render(width, height)
@@ -374,8 +382,7 @@ class Dissolve(Transition):
         else:
             rv = renpy.display.render.Render(width, height, draw_func=draw, opaque=True)
 
-        rv.focuses.extend(top.focuses)        
-        rv.depends_on(top)
+        rv.depends_on(top, True)
         rv.depends_on(bottom)
         return rv
 
@@ -581,6 +588,9 @@ class CropMove(Transition):
 
     def render(self, width, height, st, at):
 
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
+
         time = 1.0 * st / self.time
 
         # Done rendering.
@@ -618,7 +628,7 @@ class CropMove(Transition):
 # Utility function used by MoveTransition et al.
 def position(d):
 
-    xpos, ypos, xanchor, yanchor, xoffset, yoffset = d.get_placement()
+    xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel = d.get_placement()
 
     if xpos is None:
         xpos = 0
@@ -630,13 +640,13 @@ def position(d):
         yanchor = 0
 
     return xpos, ypos, xanchor, yanchor
-        
+
 # These are used by MoveTransition.
 def MoveFactory(pos1, pos2, delay, d, **kwargs):
     if pos1 == pos2:
         return d
 
-    return renpy.display.layout.Move(pos1, pos2, delay, d, **kwargs)
+    return renpy.display.motion.Move(pos1, pos2, delay, d, **kwargs)
 
 def default_enter_factory(pos, delay, d):
     return d
@@ -653,7 +663,7 @@ def MoveIn(pos, pos1, delay, d, **kwargs):
         return a
 
     pos = tuple([aorb(a, b) for a, b in zip(pos, pos1)])
-    return renpy.display.layout.Move(pos, pos1, delay, d, **kwargs)
+    return renpy.display.motion.Move(pos, pos1, delay, d, **kwargs)
 
 def MoveOut(pos, pos1, delay, d, **kwargs):
 
@@ -663,13 +673,13 @@ def MoveOut(pos, pos1, delay, d, **kwargs):
         return a
 
     pos = tuple([aorb(a, b) for a, b in zip(pos, pos1)])
-    return renpy.display.layout.Move(pos1, pos, delay, d, **kwargs)
+    return renpy.display.motion.Move(pos1, pos, delay, d, **kwargs)
 
 def ZoomInOut(start, end, pos, delay, d, **kwargs):
 
     xpos, ypos, xanchor, yanchor = pos
 
-    FactorZoom = renpy.display.layout.FactorZoom
+    FactorZoom = renpy.display.motion.FactorZoom
     
     if end == 1.0:
         return FactorZoom(start, end, delay, d, after_child=d, opaque=False,
@@ -679,10 +689,10 @@ def ZoomInOut(start, end, pos, delay, d, **kwargs):
                           xpos=xpos, ypos=ypos, xanchor=xanchor, yanchor=yanchor, **kwargs)
 
 def RevolveInOut(start, end, pos, delay, d, **kwargs):
-    return renpy.display.layout.Revolve(start, end, delay, d, pos=pos, **kwargs)
+    return renpy.display.motion.Revolve(start, end, delay, d, pos=pos, **kwargs)
     
 # TODO: Move isn't properly respecting positions when x < 0.
-def MoveTransition(delay, old_widget=None, new_widget=None, factory=None, enter_factory=None, leave_factory=None, old=False):
+def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter_factory=None, leave_factory=None, old=False, layers=[ 'master' ]):
     """
     This transition attempts to find images that have changed
     position, and moves them from the old position to the new
@@ -748,10 +758,11 @@ def MoveTransition(delay, old_widget=None, new_widget=None, factory=None, enter_
 
                 f = new.layers[layer]
 
-                if isinstance(f, renpy.display.layout.MultiBox) \
-                        and f.scene_list is not None \
-                        and layer not in renpy.config.overlay_layers:
-
+                if (isinstance(f, renpy.display.layout.MultiBox) 
+                    and layer in layers
+                    and f.scene_list is not None 
+                    and layer not in renpy.config.overlay_layers):
+                        
                     f = merge_slide(old.layers[layer], new.layers[layer])
 
                 rv.layers[layer] = f
@@ -1030,6 +1041,9 @@ class ImageDissolve(Transition):
 
     def render(self, width, height, st, at):
 
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
+
         if st >= self.time or not renpy.display.module.can_imageblend:
             self.events = True
             return render(self.new_widget, width, height, st, at)
@@ -1083,8 +1097,7 @@ class ImageDissolve(Transition):
         else:
             rv = renpy.display.render.Render(width, height, draw_func=draw, opaque=True)
 
-        rv.focuses.extend(top.focuses)        
-        rv.depends_on(top)
+        rv.depends_on(top, True)
         rv.depends_on(bottom)
         return rv
 
