@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -38,17 +38,21 @@ _return = None
 _args = None
 _kwargs = None
 
+# Should the in-game window be shown?
+_window = False
+
 # The window subtitle.
 _window_subtitle = ''
 
+# Should rollback be allowed?
+_rollback = True
+
 # config.
 _config = renpy.config
 
+
 class _Config(object):
 
-    def __init__(self, name):
-        vars(self)["_name"] = name
-    
     def register(self, name, default, cat=None, help=None):
         setattr(self, name, default)
         _config.help.append((cat, name, help))
@@ -57,7 +61,7 @@ class _Config(object):
         cvars = vars(_config)
 
         if name not in cvars:
-            raise Exception('%s.%s is not a known configuration variable.' % (self._name, name))
+            raise Exception('config.%s is not a known configuration variable.' % (name))
 
         return cvars[name]
 
@@ -65,7 +69,7 @@ class _Config(object):
         cvars = vars(_config)
 
         if name not in cvars and renpy.config.locked:
-            raise Exception('%s.%s is not a known configuration variable.' % (self._name, name))
+            raise Exception('config.%s is not a known configuration variable.' % (name))
 
         if name == "script_version":
             renpy.store._set_script_version(value)
@@ -78,8 +82,7 @@ class _Config(object):
         else:
             delattr(renpy.config, name)
         
-config = _Config("config")
-library = _Config("library")
+config = _Config()
 
 _list = list
 _dict = dict
@@ -119,6 +122,8 @@ DynamicDisplayable = renpy.display.layout.DynamicDisplayable
 ConditionSwitch = renpy.display.layout.ConditionSwitch
 ShowingSwitch = renpy.display.layout.ShowingSwitch
 
+Transform = renpy.display.motion.Transform
+
 Animation = anim.Animation
 Movie = renpy.display.video.Movie
 Particles = renpy.display.particle.Particles
@@ -128,15 +133,16 @@ Text = renpy.display.text.Text
 ParameterizedText = renpy.display.text.ParameterizedText
 
 # Currying things.
+Alpha = renpy.curry.curry(renpy.display.layout.Alpha)
 Position = renpy.curry.curry(renpy.display.layout.Position)
-Pan = renpy.curry.curry(renpy.display.layout.Pan)
-Move = renpy.curry.curry(renpy.display.layout.Move)
-Motion = renpy.curry.curry(renpy.display.layout.Motion)
-Revolve = renpy.curry.curry(renpy.display.layout.Revolve)
-Zoom = renpy.curry.curry(renpy.display.layout.Zoom)
-RotoZoom = renpy.curry.curry(renpy.display.layout.RotoZoom)
-FactorZoom = renpy.curry.curry(renpy.display.layout.FactorZoom)
-SizeZoom = renpy.curry.curry(renpy.display.layout.SizeZoom)
+Pan = renpy.curry.curry(renpy.display.motion.Pan)
+Move = renpy.curry.curry(renpy.display.motion.Move)
+Motion = renpy.curry.curry(renpy.display.motion.Motion)
+Revolve = renpy.curry.curry(renpy.display.motion.Revolve)
+Zoom = renpy.curry.curry(renpy.display.motion.Zoom)
+RotoZoom = renpy.curry.curry(renpy.display.motion.RotoZoom)
+FactorZoom = renpy.curry.curry(renpy.display.motion.FactorZoom)
+SizeZoom = renpy.curry.curry(renpy.display.motion.SizeZoom)
 Fade = renpy.curry.curry(renpy.display.transition.Fade)
 Dissolve = renpy.curry.curry(renpy.display.transition.Dissolve)
 ImageDissolve = renpy.curry.curry(renpy.display.transition.ImageDissolve)
@@ -161,6 +167,7 @@ MultiPersistent = renpy.loadsave.MultiPersistent
 
 Style = renpy.style.Style
 
+absolute = renpy.display.core.absolute
 
 def layout(cls, doc, nargs=0, **extra_kwargs):
 
