@@ -1,3 +1,2 @@
 description="A simple and complete Ren'Py game."
-ro=True
-special=True
+
